//
//  Audio_ToolController.m
//  Audio Tool
//
//  Created by John Geelen on 01-04-11.
//  Copyright 2011 JG Electronics. All rights reserved.
//	Remarks, reports to: www.jgelectronics.nl.
//
//	Derived from Apple's example program DefaultOutputUnit.
//	The application icon is from Vincent Garnier http://benjigarner.deviantart.com/art/Fruity-Apples-80805804
//
//	Cocoa, OSX10.6 and XCODE 4.0.1
//
/*
 Disclaimer: IMPORTANT: 
 This JG Electronics software is supplied to you by JG Electronics as FREE AND OPEN SOURCE SOFTWARE. 
 
 The JG Electronics Software is provided by JG Electronics on an "AS IS" basis.
 JG ELECTRONICS MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE JG ELECTRONICS SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL JG ELECTRONICS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE JG ELECTRONICS SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF JG ELECTRONICS HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

#import "Audio_ToolController.h"


@implementation Audio_ToolController

#pragma mark -
#pragma mark === Accessor Methods ===
//	*********************************************************************************************************************************
//	slider values
@synthesize freqFlt, panFlt, amplitudeFlt;

#pragma mark -
#pragma mark === Internal Methods ===
//	*********************************************************************************************************************************
- (id)init
{
	if((self = [super init]) == nil)
		return nil;
	
	gAudioToolModelPtr = [[Audio_ToolModel alloc] init];
	
	[self setFreqFlt:1200];
	[self setPanFlt:0.5];
	[self setAmplitudeFlt:0.25];
	
//	key value observing
	[self addObserver:self forKeyPath:@"freqFlt" options:NSKeyValueObservingOptionOld context:nil];
	[self addObserver:self forKeyPath:@"amplitudeFlt" options:NSKeyValueObservingOptionOld context:nil];
	[self addObserver:self forKeyPath:@"panFlt" options:NSKeyValueObservingOptionOld context:nil];
	
	return self;
}

- (void)dealloc
{
	if(gAudioToolModelPtr != nil) {
		[gAudioToolModelPtr release];
	}
	[ super dealloc ] ;
}

	#pragma mark -
	#pragma mark === Key Value Observing Methods ===
//	*********************************************************************************************************************************
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
//	NSLog(@"observeValueForKeyPath");
	if([keyPath compare:@"freqFlt"] == NSOrderedSame) {
		[gAudioToolModelPtr amSetFrequency:[self freqFlt]];
	}
	if([keyPath compare:@"panFlt"] == NSOrderedSame) {
		[gAudioToolModelPtr amSetPan:[self panFlt]];
	}
	if([keyPath compare:@"amplitudeFlt"] == NSOrderedSame) {
		[gAudioToolModelPtr amSetamplitude:[self amplitudeFlt]];
	}
}

	#pragma mark -
	#pragma mark === Action Methods ===
//	*********************************************************************************************************************************
- (IBAction)acReadButtons:(id)sender
{
	NSLog(@"acReadButtons");
	switch ([sender tag])
	{
		case 1:
		[gAudioToolModelPtr amSetamplitude:[self amplitudeFlt]];	//	restore fadervalue to amplitude
			[gAudioToolModelPtr StartAudio];
			[acStartButton setEnabled:NO];
			[acStopButton setEnabled:YES];
			break;
		case 2:
			[gAudioToolModelPtr amSetamplitude:0.0];	//	to suppress click sound when switching off audio unit
			[gAudioToolModelPtr StopAudio];
			[acStartButton setEnabled:YES];
			[acStopButton setEnabled:NO];
		break;
	}
}


@end
