//
//  Audio_ToolModel.m
//  Audio Tool
//
//  Created by John Geelen on 01-04-11.
//  Copyright 2011 JG Electronics. All rights reserved.
//	Remarks, reports to: www.jgelectronics.nl.
//
//	Derived from Apple's example program DefaultOutputUnit.
//	The application icon is from Vincent Garnier http://benjigarner.deviantart.com/art/Fruity-Apples-80805804
//
//	Cocoa, OSX10.6 and XCODE 4.0.1
//
/*
 Disclaimer: IMPORTANT: 
 This JG Electronics software is supplied to you by JG Electronics as FREE AND OPEN SOURCE SOFTWARE. 
 
 The JG Electronics Software is provided by JG Electronics on an "AS IS" basis.
 JG ELECTRONICS MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE JG ELECTRONICS SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL JG ELECTRONICS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE JG ELECTRONICS SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF JG ELECTRONICS HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

#import "Audio_ToolModel.h"

@implementation Audio_ToolModel

	#pragma mark -
	#pragma mark === Static Methods ===
//	*********************************************************************************************************************************
static OSStatus renderInput(void *inRefCon, AudioUnitRenderActionFlags *ioActionFlags, const AudioTimeStamp *inTimeStamp, UInt32 inBusNumber, UInt32 inNumberFrames, AudioBufferList *ioData);

	#pragma mark -
	#pragma mark === Internal Methods ===
//	*********************************************************************************************************************************
- (id)init
{
	if((self = [super init]) == nil)
		return nil;

/*
//	method 1 see callback method
	gSynthData.phase = 0.0;
	gSynthData.frequency = 1200;
	gSynthData.pan = 0.5;
//
*/

/*
//	method 2 see callback method
	gSynthData->phase = 0.0;
	gSynthData->frequency = 1200;
	gSynthData->pan = 0.5;
//
*/
 
//	method 3 see callback method
	phase = 0.0;
	frequency = 1200;
	pan = 0.5;
//
	
	sSampleRate = 48000;
	theFormatFlags =  kAudioFormatFlagsNativeFloatPacked | kAudioFormatFlagIsNonInterleaved;
	theBytesPerFrame = theBytesInAPacket = 4;
	theFramesPerPacket = 1;
	theBitsPerChannel = 32;
	sNumChannels = 2;
	theFormatID = kAudioFormatLinearPCM;
	
	[self CreateDefaultAU];
	[self TestDefaultAU];
	
	return self;
}

	#pragma mark -
	#pragma mark === Audio Control Methods ===
//	*********************************************************************************************************************************
- (void)amSetFrequency:(float)freqFlt {
	
//	gSynthData.frequency = freqFlt;		//	method 1
//	gSynthData->frequency = freqFlt;	//	method 2
	frequency = freqFlt;				//	method 3
}

- (void)amSetPan:(float)panFlt {
//	gSynthData.pan = panFlt;		//	method 1
//	gSynthData->pan = panFlt;		//	method 2
	pan =  panFlt;					//	method 3

}

- (void)amSetamplitude:(float)amplFlt {
	AudioUnitSetParameter(gOutputUnit, kHALOutputParam_Volume, kAudioUnitScope_Output, 0, amplFlt, 0);
}

	#pragma mark -
	#pragma mark === Audio Methods ===
//	*********************************************************************************************************************************
- (void)CreateDefaultAU		//	Create Audio Unit Defaultoutput
{
	NSLog(@"Start CreateDefaultAU");
	OSStatus err = noErr;
	
//	Open the default output audio unit
//	ComponentDescription			desc;
	AudioComponentDescription		desc;
	desc.componentType =			kAudioUnitType_Output;
	desc.componentSubType =			kAudioUnitSubType_DefaultOutput;
	desc.componentManufacturer =	kAudioUnitManufacturer_Apple;
	desc.componentFlags =			0;
	desc.componentFlagsMask =		0;
	
//	Component comp = FindNextComponent(NULL, &desc);
	AudioComponent comp = AudioComponentFindNext(NULL, &desc);
	if (comp == NULL)
		{
		NSLog(@"component not found");
		return;
		}
	
//	err = OpenAComponent(comp, &gOutputUnit);
	err = AudioComponentInstanceNew(comp, &gOutputUnit);
	if (comp == NULL) {
		NSLog(@"OpenAComponent=%d", err);
		return;
	}
	
//	Set up a callback function to generate output to the output unit
	AURenderCallbackStruct		input;
	input.inputProc =			renderInput;
//	input.inputProcRefCon =		&gSynthData;	//	method 1 and 2
	input.inputProcRefCon =		self;			//	method 3
	
	err = AudioUnitSetProperty (gOutputUnit,
								kAudioUnitProperty_SetRenderCallback, 
								kAudioUnitScope_Input,
								0, 
								&input, 
								sizeof(input));
	if (err) {
		NSLog(@"AudioUnitSetProperty-CB=%d", err);
		return;
	}
	
	AudioUnitSetParameter(gOutputUnit, kHALOutputParam_Volume, kAudioUnitScope_Output, 0, 1.0, 0);
}

- (void)TestDefaultAU
{
	OSStatus err = noErr;
//	We tell the Audio Unit Defaultoutput what data format we're going to supply to it,
//	this is necessary if you're providing data through an input callback
//	and you want the Audio Unit Defaultoutput to do any format conversions necessary from your format to the device's format.
	AudioStreamBasicDescription streamFormat;
	streamFormat.mSampleRate = sSampleRate;					//	the sample rate of the audio stream = 48000
	streamFormat.mFormatID = theFormatID;					//	the specific encoding type of audio stream = kAudioFormatFlagsNativeFloatPacked | kAudioFormatFlagIsNonInterleaved
	streamFormat.mFormatFlags = theFormatFlags;				//	flags specific to each format
	streamFormat.mBytesPerPacket = theBytesInAPacket;		//	4
	streamFormat.mFramesPerPacket = theFramesPerPacket;		//	1
	streamFormat.mBytesPerFrame = theBytesPerFrame;			//	4
	streamFormat.mChannelsPerFrame = sNumChannels;			//	2
	streamFormat.mBitsPerChannel = theBitsPerChannel;		//	32
	
	NSLog(@"Rendering source:\n\t");
	NSLog(@"SampleRate=%f,", streamFormat.mSampleRate);
	NSLog(@"BytesPerPacket=%u,", streamFormat.mBytesPerPacket);
	NSLog(@"FramesPerPacket=%u,", streamFormat.mFramesPerPacket);
	NSLog(@"BytesPerFrame=%u,", streamFormat.mBytesPerFrame);
	NSLog(@"BitsPerChannel=%u,", streamFormat.mBitsPerChannel);
	NSLog(@"ChannelsPerFrame=%u\n", streamFormat.mChannelsPerFrame);
	
	err = AudioUnitSetProperty (gOutputUnit, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Input, 0, &streamFormat, sizeof(AudioStreamBasicDescription));
	if (err) {
		NSLog(@"AudioUnitSetProperty-SF=%4.4s, %d", (char*)&err, err);
		return;
	}
	
// Initialize unit
	err = AudioUnitInitialize(gOutputUnit);
	if (err) {
		NSLog(@"AudioUnitInitialize=%d", err);
		return;
	}
    
	Float64 outSampleRate = 48000;
	UInt32 size = sizeof(Float64);
	err = AudioUnitGetProperty (gOutputUnit, kAudioUnitProperty_SampleRate, kAudioUnitScope_Output, 0, &outSampleRate, &size);
	if (err) {
		NSLog(@"AudioUnitSetProperty-GF=%4.4s, %d", (char*)&err, err);
		return;
	}
	
//	Start the rendering
//	The DefaultOutputUnit will do any format conversions to the format of the default device
	
//	we call the CFRunLoopRunInMode to service any notifications that the audio system has to deal with
	CFRunLoopRunInMode(kCFRunLoopDefaultMode, 2, false);
}

- (void)StartAudio
{
	NSLog(@"StartAudio");
	OSStatus err = noErr;
	
	err = AudioOutputUnitStart(gOutputUnit);
	if (err) {
		NSLog(@"AudioOutputUnitStart error");
		return;
	}
}

- (void)StopAudio
{	
	NSLog(@"StopAudio");
	OSStatus err = noErr;
	
	err = AudioOutputUnitStop(gOutputUnit);
	if (err) {
		printf ("AudioOutputUnitStop=%d\n", err); return;
	}
}


- (void)CloseDefaultAU
{	CloseComponent(gOutputUnit); }

	#pragma mark -
	#pragma mark === Callback Procedures ===
//	*********************************************************************************************************************************
static OSStatus renderInput(void *inRefCon, AudioUnitRenderActionFlags *ioActionFlags, const AudioTimeStamp *inTimeStamp, UInt32 inBusNumber, UInt32 inNumberFrames, AudioBufferList *ioData)
{
//	NSLog(@"audio render procedure");
//	To explain the access to the data in the render call back here 3 different ways how to get access to Sinewave's data with 3 different methods	
/*						
 SynthData& sinData = *(SynthData*)inRefCon;					//	method 1
	float			freq = sinData.frequency;					//	method 1
	float			pan = sinData.pan;							//	method 1
	float			phase = afr.phase;							//	method 1
*/
//	SynthData *		sinData = (SynthData *)inRefCon;			//	method 2
	Audio_ToolModel * sinData = (Audio_ToolModel *)inRefCon;	//	method 3
	float			freq = sinData->frequency;					//	method 2 and 3
	float			pan = sinData->pan;							//	method 2 and 3
	float			phase = sinData->phase;						//	method 2 and 3

//	private parameters
	UInt32			frame;
	Float32			nextSample;
	float			phaseIncrement = 2 * M_PI * freq / 48000.0;
//	Get a pointer to the dataBuffer of the AudioBufferList
	float *			outA = (float*)ioData->mBuffers[0].mData;
	float *			outB = (float*)ioData->mBuffers[1].mData;
	
	for (frame = 0; frame < inNumberFrames; ++frame) {
		nextSample = sin(phase);					//	* amplitude = 1;
		outA[frame] = nextSample * pan;				//	left channel
		outB[frame] = nextSample * (1.0 - pan);		//	right channel
		phase = phase + phaseIncrement;
		if (phase >= 2 * M_PI) {
			phase = phase - 2 * M_PI;
		}
	}
//	sinData.phase = phase;		//	method 1
	sinData->phase = phase;		//	method 2 and 3
	return noErr;
}


@end
